/*
 * SPDX-FileCopyrightText: 2018-2024 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include <config.h>
#include "rugby-possibility.h"

struct _RugbyPossibility
{
    GObject parent_instance;

    int tries;
    int utries;
    int kicks;
};

G_DEFINE_FINAL_TYPE (RugbyPossibility, rugby_possibility, G_TYPE_OBJECT)

enum
{
    PROP_TRIES = 1,
    PROP_UTRIES,
    PROP_KICKS,
    N_PROPS
};

static GParamSpec *properties[N_PROPS];

static void
rugby_possibility_get_property (GObject    *object,
                                unsigned    prop_id,
                                GValue     *value,
                                GParamSpec *pspec)
{
    RugbyPossibility *self = RUGBY_POSSIBILITY (object);

    switch (prop_id)
      {
      case PROP_TRIES:
          g_value_set_int (value, rugby_possibility_get_tries (self));
          break;
      case PROP_UTRIES:
          g_value_set_int (value, rugby_possibility_get_utries (self));
          break;
      case PROP_KICKS:
          g_value_set_int (value, rugby_possibility_get_kicks (self));
          break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      }
}

static void
rugby_possibility_set_property (GObject      *object,
                                unsigned      prop_id,
                                const GValue *value,
                                GParamSpec   *pspec)
{
    RugbyPossibility *self = RUGBY_POSSIBILITY (object);

    switch (prop_id)
      {
      case PROP_TRIES:
          self->tries = g_value_get_int (value);
          break;
      case PROP_UTRIES:
          self->utries = g_value_get_int (value);
          break;
      case PROP_KICKS:
          self->kicks = g_value_get_int (value);
          break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      }
}

static void
rugby_possibility_class_init (RugbyPossibilityClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->get_property = rugby_possibility_get_property;
    object_class->set_property = rugby_possibility_set_property;

    properties[PROP_TRIES] =
        g_param_spec_int ("tries", "", "",
                          0, 200, 0,
                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
    properties[PROP_UTRIES] =
        g_param_spec_int ("utries", "", "",
                          0, 200, 0,
                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
    properties[PROP_KICKS] =
        g_param_spec_int ("kicks", "", "",
                          0, 200, 0,
                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);

    g_object_class_install_properties (object_class,
                                       N_PROPS,
                                       properties);

}

static void
rugby_possibility_init (G_GNUC_UNUSED RugbyPossibility *self)
{
}

RugbyPossibility *
rugby_possibility_new (int tries,
                       int utries,
                       int kicks)
{
    return g_object_new (RUGBY_TYPE_POSSIBILITY,
                         "tries", tries,
                         "utries", utries,
                         "kicks", kicks,
                         NULL);
}

int
rugby_possibility_get_tries (RugbyPossibility *self)
{
    g_assert (RUGBY_IS_POSSIBILITY (self));

    return self->tries;
}

int
rugby_possibility_get_utries (RugbyPossibility *self)
{
    g_assert (RUGBY_IS_POSSIBILITY (self));

    return self->utries;
}

int
rugby_possibility_get_kicks (RugbyPossibility *self)
{
    g_assert (RUGBY_IS_POSSIBILITY (self));

    return self->kicks;
}

int
rugby_possibility_total_tries (RugbyPossibility *self)
{
    g_assert (RUGBY_IS_POSSIBILITY (self));

    return self->tries + self->utries;
}
